# -*- coding: utf-8 -*-

import os
import platform
import sys
import tempfile
import tarfile

base = os.getcwd()

PYTHON_VERSION = ".".join((str(sys.version_info[0]), str(sys.version_info[1])))
OS_VERSION = tuple(int(v) for v in platform.mac_ver()[0].split('.'))

# distribution version
DISTRIBUTION_VERSION = 1

# dependencies name and version
bzr = {
    'name':'bzr',
    'version':'2.6b1',
    'destdir':'bzr',
    'url':'http://launchpad.net/bzr/%(version).3s/%(version)s/+download/bzr-%(version)s.tar.gz',
    'patches':(
#        ('texinfo_pdb.patch', 0),
#        ('rename_texinfo_builder.patch', 0),
        ('cacert.patch', 0),
        ('generate_docs.patch', 0),
    ),
}

# packages
dulwich = {
    'name':'dulwich',
    'version':'0.8.3',
    'destdir':'dulwich',
    'url':'http://www.samba.org/~jelmer/dulwich/dulwich-%(version)s.tar.gz'
}
launchpadlib = {
    'name':'launchpadlib',
    'version':'1.9.12',
    'destdir':'launchpadlib',
    'url':'http://launchpad.net/launchpadlib/trunk/%(version)s/+download/launchpadlib-%(version)s.tar.gz',
}
paramiko = {
    'name':'paramiko',
    'version':'1.7.7.1',
    'destdir':'paramiko',
    'url':'http://www.lag.net/paramiko/download/paramiko-%(version)s.tar.gz',
}
pycrypto = {
    'name':'pycrypto',
    'version':'2.4.1',
    'destdir':'pycrypto',
    'url':'http://www.pycrypto.org/files/pycrypto-%(version)s.tar.gz',
}
python_fastimport = {
    'name':'python-fastimport',
    'version':'0.9.0',
    'destdir':'python-fastimport',
    'url':'http://launchpad.net/python-fastimport/trunk/%(version)s/+download/python-fastimport-%(version)s.tar.gz',
}
pyqt = {
    'name':'PyQt-mac-gpl',
    'version':'4.8.6',
    'destdir':'pyqt',
    'url':'http://www.riverbankcomputing.co.uk/static/Downloads/PyQt4/PyQt-mac-gpl-%(version)s.tar.gz',
}
sip = {
    'name':'sip',
    'version':'4.13',
    'destdir':'sip',
    'url':'http://www.riverbankcomputing.co.uk/static/Downloads/sip4/sip-%(version)s.tar.gz',
}
subvertpy = {
    'name':'subvertpy',
    'destdir':'subvertpy',
    'branch':'lp:subvertpy',
    'revno':'tag:subvertpy-0.8.10',
}

# plugins
bzrtools = {
    'name':'bzrtools',
    'destdir':'bzrtools',
    'branch':'lp:bzrtools',
    'revno':'tag:release-2.5',
}
colo = {
    'name':'bzr-colo',
    'destdir':'colo',
    'branch':'lp:bzr-colo',
    'version':'0.4.0',
    'url':'http://launchpad.net/bzr-colo/trunk/%(version)s/+download/bzr-colo-%(version)s.tar.gz',
}
email = {
    'name':'bzr-email',
    'destdir':'email',
    'branch':'lp:bzr-email',
    'revno':'52',
}
explorer = {
    'name':'bzr-explorer',
    'destdir':'explorer',
    'branch':'lp:bzr-explorer',
    'revno':'tag:release-1.2.2',
}
fastimport = {
    'name':'bzr-fastimport',
    'destdir':'fastimport',
    'branch':'lp:bzr-fastimport',
    'revno':'tip',
}
git = {
    'name':'bzr-git',
    'destdir':'git',
    'branch':'lp:bzr-git',
    'revno':'tag:bzr-git-0.6.7',
}
keychain = {
    'name':'bzr-keychain',
    'destdir':'keychain',
    'branch':'lp:bzr-keychain',
    'revno':'15',
}
loom = {
    'name':'bzr-loom',
    'destdir':'loom',
    'branch':'lp:bzr-loom',
    'revno':'150',
}
pipeline = {
    'name':'bzr-pipeline',
    'destdir':'pipeline',
    'branch':'lp:bzr-pipeline/stable',
    'revno':'tag:release-1.4',
}
qbzr = {
    'name':'qbzr',
    'destdir':'qbzr',
    'branch':'lp:qbzr/0.22',
    'revno':'tag:release-0.22.1',
}
rewrite = {
    'name':'bzr-rewrite',
    'destdir':'rewrite',
    'branch':'lp:bzr-rewrite',
    'revno':'tip',
}
svn = {
    'name':'bzr-svn',
    'destdir':'svn',
    'branch':'lp:bzr-svn',
    'revno':'tag:bzr-svn-1.2.1',
}
upload = {
    'name':'bzr-upload',
    'destdir':'upload',
    'branch':'lp:bzr-upload/1.1',
    'revno':'tag:bzr-upload-1.1.0',
}
xmloutput = {
    'name':'bzr-xmloutput',
    'destdir':'xmloutput',
    'branch':'lp:bzr-xmloutput',
    'revno':'161',
}

# list of packages and plugins to build
# build in order as they appear here
packages = [
    paramiko,
    pycrypto,
    bzr,
    sip,
    pyqt,
    launchpadlib,
    python_fastimport,
    dulwich,
]
plugins = [
    bzrtools,
    colo,
    email,
    explorer,
    fastimport,
    git,
    loom,
    pipeline,
    qbzr,
    rewrite,
    upload,
    xmloutput,
]

docs = {
    'doc/developers/_build/latex': {
        'bzr-en-architecture-overview.pdf'      : 'Developers/Architecture Overview.pdf',
        'bzr-en-developer-guide.pdf'            : 'Developers/Developer Guide.pdf',
        'bzr-en-integration-guide.pdf'          : 'Developers/Integration Guide.pdf',
        'bzr-en-testing-guide.pdf'              : 'Developers/Testing Guide.pdf',
    },
    'doc/en/_build/latex': {
        'bzr-en-admin-guide.pdf'                : "English/Administrator's Guide.pdf",
        'bzr-en-tutorial-centralized.pdf'       : 'English/Centralized Workflow Tutorial.pdf',
        'bzr-en-tutorial-mini.pdf'              : 'English/Bazaar in Five Minutes.pdf',
        'bzr-en-tutorial-with-launchpad.pdf'    : 'English/Using Bazaar with Launchpad.pdf',
        'bzr-en-tutorial.pdf'                   : 'English/Tutorial.pdf',
        'bzr-en-upgrade-guide.pdf'              : 'English/Upgrade Guide.pdf',
        'bzr-en-user-guide.pdf'                 : 'English/User Guide.pdf',
        'bzr-en-user-reference.pdf'             : 'English/User Reference.pdf',
        'bzr-en-whats-new.pdf'                  : "English/What's New.pdf",
    },
    'doc/en/_build/html': {
        '.'                                     : 'English/html',
    },
    'doc/en/_static/en': {
        'bzr-en-quick-reference.pdf'            : 'English/Quick Reference.pdf',
    },
    'doc/es/_build/latex': {
        'bzr-es-tutorial-mini.pdf'              : 'Español/Bazaar en Cinco Minutos.pdf',
        'bzr-es-user-guide.pdf'                 : 'Español/Guia de Usuario.pdf',
    },
    'doc/es/_build/html': {
        '.'                                     : 'Español/html',
    },
    'doc/es/_static/es': {
        'bzr-es-quick-reference.pdf'            : 'Español/Referencia rápida.pdf',
    },
    'doc/ru/_build/html': {
        '.'                                     : 'Русский/html',
    },
    'doc/ru/_static/ru': {
        'bzr-ru-quick-reference.pdf'            : 'Русский/Краткий справочник.pdf',
    },
    'doc/ja/_build/html': {
        '.'                                     : '日本語/html',
    },
}

## end of configurable stuff ##

if OS_VERSION[1] >= 5:
    plugins.extend([svn, keychain])
    packages.append(subvertpy)

# utilities
def extract(src, dest):
    tmpdir = tempfile.mkdtemp()
    tar = tarfile.open(src)
    tar.extractall(tmpdir)
    tar.close()

    tmp = None
    for entry in os.listdir(tmpdir):
        if os.path.isdir(os.path.join(tmpdir,entry)):
            tmp = entry
            break

    if tmp is None:
        raise Exception('tarfile %s is empty' % src)
    tmp_dest = os.path.join(tmpdir, dest)
    os.rename(os.path.join(tmpdir, tmp), tmp_dest)
    return tmp_dest

def get_plugin_fullname(plugin):
    name = plugin['name']
    version = None
    if plugin.get('version', None) is None:
        if plugin['revno'] == 'tip':
            version = 'tip'
        elif plugin['revno'][:4] == 'tag:':
            version = plugin['revno'][4:]
        else:
            version = 'r' + plugin['revno']
    else:
        version = plugin['version']
    return '%s-%s' % (name, version)

def get_tarfile(plugin):
    return get_plugin_fullname(plugin) + '.tar.gz'
