#!/usr/bin/env python
from optparse import OptionParser
import os

import bzrlib
from bzrlib import errors
from bzrlib.workingtree import WorkingTree

import download


def get_tree_revno(tree_dir):
    wt = None
    try:
        try:
            wt = WorkingTree.open_containing(tree_dir)[0]
            wt.lock_read()
        except (errors.NoWorkingTree, errors.NotLocalUrl):
            print >> sys.stderr, "Error: no working tree at %s" % tree_dir
            return 'unknown'
        revid = wt.last_revision()
        try:
            revno_t = wt.branch.revision_id_to_dotted_revno(revid)
        except errors.NoSuchRevision:
            print >> sys.stderr, "Error: could not map rev id '%s' to revno" % revid
            return 'unknown'
        revno = ".".join(str(n) for n in revno_t)
        return revno
    finally:
        if wt is not None:
            wt.unlock()


if __name__ == '__main__':
    parser = OptionParser(usage="%prog [-c CONFIG_FILE]") 
    parser.add_option('-c', '--config', dest='config', action='store', default='config')
    (options, args) = parser.parse_args()

    try:
        global config
        config = __import__(options.config, globals(), locals(), [], 0)
    except ImportError:
        print "ERROR: failed to import config file"
        sys.exit(1)
    
    external_dir = os.path.join(config.base, 'src')
    
    with bzrlib.initialize():
        all = config.packages + config.plugins
        for dep in all:
            download.resolve_vars(dep)
            version = dep.get('version', None)
            if version == None:
                if dep['revno'] == 'tip':
                    tree_dir = os.path.join(external_dir, dep['name'])
                    version = 'r%s (tip)' % get_tree_revno(tree_dir)
                else:
                    if dep['revno'][:4] == 'tag:':
                        version = dep['revno'][4:]
                    else:
                        version = 'r' + dep['revno']
            print("%s %s" % (dep['name'], version))